
# This is the android file transfer interface.  Using adb
# it makes the andriod devices files available.

package require Expect

# The open request.  We have host, user, password & port 
# host and ftpI are usually the same and MUST not be ""
# possible connection modes are:
# option                       <user> value
# USB      (-d adb option)     usb -nocase  i.e. -nocase *usb*
# emulator (-e adb option)     emu -nocase  i.e. -nocase *emu*
# serial # (-s adb option)     ser -nocase  i.e. -nocase *ser*
#   serial implies usb              in this case <host> is taken 
#                                   as the serial #
# product  (-p adb option)     pro  -nocase  
#                                   in this case <host> is taken 
#                                   as the product id (-p argument)
# connect <host>[:<port>]      con -nocase  i.e. -nocase *con*
#                          we will get port 22 if the user has not
#                          set it.  We morf this to 5555 otherwise
#                          we use what was passed.
# How we decode all this.. we use the <user> passed in to determine
# how to connect.  If <password> contains 'su' root access is used 
# on the device (adb shell;su etc.). 

proc aFTPopen {ftpI host user password port rq_timeout args } {
  global ftp glob
  upvar #0 ftp($ftpI,timeout) timeout
  upvar #0 ftp($ftpI,handle) spawn_id
  frputs "aFTP_OpenLink $ftpI $host $user $password $port $rq_timeout "
#  log_file -a aFTP-EXP-LOG
  set simp 0
  switch -nocase -glob $user {
    "*usb*" {set op "-d"}
    "*emu*" {set op "-e"}
    "*ser*" {set op "-s $host"}
    "*pro*" {set op "-p $host"}
    "*con*" {
      set addr $host:[expr {$port == 22 ? "5555" : $port}]
      set op "connect $addr";
      set simp 1
    }
    default {return -code error "user name ($user) must be one of\
                                 'usb' 'emu' 'ser' 'pro' or 'con'"}
  }
  set ftp($ftpI,mode) [expr {[regexp -nocase {su} $password] ? "su" : ""}]
  set timeout $rq_timeout
  set ftp($ftpI,handle) -1
  set ftp($ftpI,wd) "/"
  set ftp(echo) [expr {$glob(debug) ? "" : "-noecho"}]
  # pelim done, lets see if we can talk...
  FTPlog_user
  if {$simp} {
    # This is a network connection...
    eval "spawn $ftp(echo) adb disconnect $addr"
    expect eof
    frputs "Try connect: adb " op
    eval "spawn $ftp(echo) adb $op"
    expect \
	-re "(unable to connect .*)\r\n" {incr re 0} \
	-re ".*connected to (\[^\r]*).*\n" {incr re } \
	eof {set expect_out(1,string) "eof"; incr re 0}\
	timeout {set expect_out(1,string) "timeout"; incr re 0}
    expect eof
    catch exp_close
    frputs "re" " and " "expect_out(1,string)"
    if {![info exists re] || ! $re} {
      return -code error "$expect_out(1,string) Is device connected?"
    }
    set ftp($ftpI,serialno) $expect_out(1,string)
    aFTPstartShell $ftpI
    return 0
  }
  # This is NOT a network connection
  catch exp_close
  eval "spawn $ftp(echo) adb $op get-serialno"
  FTPlog_user
  set re 0
  expect \
      -re "(error:\[^\r]*)\r\n" {incr re 3}\
      -re "(unknown\[^\r]*)\r\n" {incr re 2} \
      -re "(\[^\r]*)\r\n" {incr re} \
      eof
  catch exp_close
  switch $re {
    0 {return -code error "No response. Is device connected?" }
    1 {
      set ftp($ftpI,serialno) $expect_out(1,string)
      aFTPstartShell $ftpI
      return}
    2 {return -code error "$expect_out(1,string)\
                           May need to start server from root" }
    3 {return -code error $expect_out(1,string) }
  }
}

proc aFTPstartShell {ftpI } {
  global ftp
  upvar #0 ftp($ftpI,handle) spawn_id
  eval "spawn $ftp(echo) adb -s $ftp($ftpI,serialno) shell"
  expect \
      -re "(error:\[^\r]*)\r\n" {incr re 2}\
      -re "(\[^ ]* )\$" {incr re} \
      eof {incr re 0}
  switch $re {
    0 {catch exp_close ;return -code error "Unexpected close of connection"}
    2 {catch exp_close ;return -code error $expect_out(1,string)}
    3 {set ftp($ftpI,prompt) $expect_out(1,string)}
  }
  if { $ftp($ftpI,mode) != "" } {
    exp_send "su\r"
    set re 0
    expect \
	-re "su\r*\n" {exp_continue}\
	-re "(error:\[^\r]*)\r\n" {incr re 2}\
	-re "(\[^ ]* )\$" {incr re} \
	eof {incr re 0}
    frputs "Sent su and got " re expect_out(1,string)
    switch $re {
      0 {catch exp_close ;return -code error "Unexpected close of connection"}
      2 {catch exp_close ;return -code error $expect_out(1,string)}
      1 {set ftp($ftpI,prompt) $expect_out(1,string)}
    }
  }
  catch  {exp_match_max 1000000}

  frputs "Start Shell " ftp($ftpI,prompt) ftpI
}

proc aFTPstopShell {ftpI} {
  global ftp
  upvar #0 ftp($ftpI,handle) spawn_id
  exp_send "exit\r"
  catch exp_close
}
  
proc aFTPsingle {ftpI what file} {
  global ftp
  upvar #0 ftp($ftpI,handle) spawn_id
  upvar #0 ftp($ftpI,timeout) timeout
  # This routine expects the adb connection to be in shell mode
  # This does a rather standard request and waits for a response.
  # While the title is 'single' one can pass more than one thing
  # in 'what'.  The eval will do the right thing by spaces 
  # (unless you want to save them, in which case you will want
  # to use FixFileName (likely level 3) like we do for 'file').

  # This routine returns any response other than a 'prompt' which gets
  # a null return.
  # eof and timeout throw errors.

  set file [FixFileName $file 1 "& "]
  frputs "try " what file 
  exp_send "$what $file\r"
  FTPlog_user

  expect \
      -re "$what $file\r*\n?" {exp_continue} \
      -re "^$ftp($ftpI,prompt)\$" {incr re 0} \
      -re "\r*\n?(.*)\r+\n$ftp($ftpI,prompt)\$" {incr re} \
      -re "\r*\n?(.*)$ftp($ftpI,prompt)\$" {incr re} \
      eof {set expect_out(1,string) "eof";incr re 2} \
      timeout {set expect_out(1,string) "timeout";incr re 3}
  frputs "single result: " "expect_out(buffer)" \
           "expect_out(1,string)" "re"
  switch $re {
    0 {return ""}
    1 {return  [regsub -all {\r} $expect_out(1,string) {}]}
    default {return -code error  [regsub -all {\r} $expect_out(1,string) {}]}
  }
}

proc aFTPcd {ftpI new_wd} {
  global ftp

  set re [aFTPsingle $ftpI  cd $new_wd]
  if {$re == ""} {
    set ftp($ftpI,wd)  $new_wd
    return 1
  }
  return -code error $re
}

proc aFTPrename {ftpI  old new} {
  global ftp
  # current name old change to new
  set re [aFTPsingle $ftpI  "mv [FixFileName $old 1 {& }]" $new] 
  if {$re != "" } {
    return -code error $re
  }
  return ""
}

proc aFTPdelete {ftpI filename} {
  global ftp
  # file to remove filename

  return [aFTPsingle $ftpI  rm $filename] 
}

proc aFTPmkdir { ftpI dir } {
  global ftp
  # create dir ftp($ftpI,mkdir,dir)
  set re [aFTPsingle $ftpI  mkdir $dir] 
  if {$re != "" } {
    return -code error $re
  }
  return 0
}

proc aFTPrmdir { ftpI dir } {
  global ftp
  # dir to remove is ftp($ftpI,rmdir,dir)

  set re [aFTPsingle $ftpI  rmdir $dir] 
  if {$re != "" } {
    return -code error $re
  }
  return 0
}

proc aFTPpwd { ftpI } {
  global ftp

  return [aFTPsingle $ftpI "pwd" ""]
}


proc aFTPlist { ftpI all} {
  global ftp
  # use -l (long list) -f (no sort) -a depends on all
  # about 12.5k entries at 80 chars each (hay, just poke around on sourceforge)
  set cmd [expr {$all == "-a" ? "ls -a -l" : "ls -l"}]
  return [split [aFTPsingle $ftpI $cmd ""] "\n" ]
}


proc aFTP_DoSearch { ftpI filename } {
  global ftp
  upvar #0 ftp($ftpI,timeout) timeout
  return -code error "Not supported ... yet"
}

proc aFTPreget { ftpI remoteFileName localFileName } {
  return [aFTPget $ftpI $remoteFileName $localFileName]
}

proc aFTPget { ftpI remoteFileName localFileName } {
  global ftp
  upvar #0 ftp($ftpI,timeout) timeout
  # get  remoteFileName and put it localFileName
  # use -p to get full times etc.
  set remotename [regsub -all " " $remoteFileName {\\ }]
  frputs "try get $remoteFileName  [FixFileName $localFileName 1 {& }]"
  eval "spawn $ftp(echo) adb -s $ftp($ftpI,serialno)\
                     pull $remotename \
                           [FixFileName $localFileName 1 {& }]"
  FTPlog_user

  expect \
      -re {(failed to .*)\r+\n} \
      {incr re;frputs "expect_out(1,string)"} \
      -re {(remote object .*)\r+\n} \
      {incr re 2;frputs "expect_out(1,string)"} \
      -re {\r*(.*bytes in *\[^\r]*\r+\n)} \
      {incr re 3;frputs "expect_out(1,string)"} \
      eof {incr re 0}
  frputs "re" "expect_out(1,string)"
  if {$re == 1 && $ftp($ftpI,mode) == "su"} {
  }
  if {$re} {return -code error "$expect_out(1,string)"}
  return $re
}

proc aFTPput { ftpI localFileName remoteFileName } {
  global ftp
  upvar #0 ftp($ftpI,timeout) timeout
  frputs "try put $localFileName to $remoteFileName "
  set localname [regsub -all " " $localFileName {\\ }]
#
  eval "spawn $ftp(echo) adb -s $ftp($ftpI,serialno)\
                      push $localname [FixFileName $remoteFileName 1 {& }]"
  FTPlog_user
  set re 0
  expect \
      -re "^(failed to copy .*)\r+\n" {incr re} \
      -re "^(Couldn't .*)\r+\n" {incr re} \
      -re "Fetching .*\r+\n" {exp_continue} \
      -re "\r.*$remoteFileName *(\[^ ].*:\[0-9]\[0-9])" {incr re 0}\
      -re "^\r+\n" {incr re 0}
  if {$re} {return -code error "$expect_out(1,string)"}
  return $re
}

proc aFTPlink { ftpI  exist new} {
  global ftp
  upvar #0 ftp($ftpI,timeout) timeout
  frputs "try ln $exist $new "
  set re [aFTPsingle $ftpI "ln [FixFileName $exist 1 {& }]" $new]
  if {$re == ""} {
    return ""
  }
  return -code error $re
  # point to this: ftp($ftpI,exist_fname) with this name ftp($ftpI,ln_fname)
  exp_send "ln [regsub -all {\ } $exist {\\ }] \
               [regsub -all {\ } $new {\\ }]\n"
  expect \
      -re "Permission denied\r\nsftp>" {incr re} \
      -re "Couldn't .*\r\nsftp>" {incr re} \
      -re ".*\r\nsftp>" {incr re 0}
  if {$re} {return -code error "$expect_out(1,string)"}
  return $re
}

proc aFTPchmod {ftpI mode file} {
  global ftp
  upvar #0 ftp($ftpI,timeout) timeout
  FTPlog_user
  frputs "aFTPchmod trying " ftpI mode file
  return [aFTPsingle $ftpI "chmod $mode" $file]
}

proc aFTPchown {ftpI mode file} {
  global ftp
  upvar #0 ftp($ftpI,timeout) timeout
  FTPlog_user
  frputs "aFTPchown trying " ftpI mode file
  return [aFTPsingle $ftpI "chown $mode" $file]
}

proc aFTPclose { ftpI} {
  global ftp
  upvar #0 ftp($ftpI,handle) spawn_id
  set spawn_id -1
}


proc aFTPcommand {ftpI command} {
  global ftp glob
  upvar #0 ftp($ftpI,handle) spawn_id
  upvar #0 ftp($ftpI,timeout) timeout
  frputs "try $command " spawn_id
  FTPlog_user
#  expect -re ".*"
#  Log "sendingfrom pwd: pwd "
  return [aFTPsingle $ftpI $command ""]
  eval "spawn $ftp(echo) adb $command"
#  exp_send "$command\r"
  set re 0
  expect  -re "(.*)\r+\n.?> " {incr re 2}\
      -re "(.*)\r+\n.?" {incr re 4} \
      eof {incr re} \
      timeout {set expect_out(1,string) "timeout";incr re 3}
  frputs "command " re expect_out(1,string) 
  catch exp_close

  return [regsub -all {\r} $expect_out(1,string) {}]
}
