#
# this package provides 'balloon help' AKA tooltip help
# for tk windows. It uses several globals to control
# the details of font, colors, delay and enabling.
#
# To define a 'tip' for a window include one of the following
# lines in the code definition:
#
# balloonhelp_for window text
#
# balloonhelpd_for window text
#
# balloonhelp_for will pop up the tip window after a delay
# balloonhelpd_for will pop up the window imeadiatly
# 
# In both cases the trigger for the display is the mouse
# entering the window and the tear down is triggered by
# the mouse leaveing the window
#
# The following globals may be used to control balloon help:
#
#  ::balloon_help::font
#  ::balloon_help::fg
#  ::balloon_help::bg
#  ::balloon_help::enable       (1 do the tip, 0 disable tips)
#  ::balloon_help::delay        (delay time in ms for the delay version)
#
#  If one of ::balloon_help::font,  ::balloon_help::fg, ::balloon_help::bg
#
#  is changed balloon_help_config may be called to update the display
#
#  An alternate way to change the font or color is to call
#       balloon_help_config option 'new option value'
#
#  where option is one of {font bg fg delay} any number of option value
#  pairs may be passed in the call
#
#Set up balloon help
option add *Balloonhelp*background white widgetDefault
option add *Balloonhelp*foreground black widgetDefault
option add *Balloonhelpinfo.wrapLength 3i  widgetDefault
option add *Balloonhelp.info.justify left widgetDefault
#set ColorSpecs(BalloonHelp,Background) \#FA4
#set ColorSpecs(BalloonHelp,Foreground) black
#set ColorSpecs(Default,Background) \#FFD8B1

proc balloonhelp_for  { a b} {::balloon_help::balloonhelp_for $a $b}
proc balloonhelpd_for { a b} {::balloon_help::balloonhelpd_for $a $b}
proc balloon_help_config {args} {eval "::balloon_help::balloon_help_config $args"}

namespace eval balloon_help {
  variable font
  variable fg
  variable bg
  variable delay 2000
  variable enable 0

  namespace export balloon_help_config balloonhelp_for balloonhelpd_for

  proc balloon_help_config { args} {
    variable font
    variable fg
    variable bg
    foreach {arg val} $args {
      set $arg $val
    }
    foreach op {font fg bg} {
      if {[info exists $op]} {
	.balloonhelp.info config -$op [set $op]
      }
    }
  }
  
  toplevel .balloonhelp \
      -class Balloonhelp \
      -background black \
      -borderwidth 1 \
      -relief flat
  #label .balloonhelp.arrow -anchor nw -bitmap @arrow.xbm
  #pack .balloonhelp.arrow -side left -fill y
  label .balloonhelp.info 
  balloon_help_config
  pack .balloonhelp.info -side left -fill y
  wm overrideredirect .balloonhelp 1
  wm withdraw .balloonhelp
  .balloonhelp.info configure -text ""

  #set config(balloonhelp) 1
  array set OverlapP {}

  proc balloonhelp_for {win mesg} {
    variable bhInfo
    set ::bhOverlapP($win) 1; 
    set bhInfo($win) $mesg
    bind $win <Enter>  +[namespace code {balloonhelp_pending %W}]
    bind $win <Leave>  +[namespace code {balloonhelp_cancel}]
  }

  proc balloonhelpd_for {win mesg} {
    variable bhInfo
    set ::bhOverlapP($win,d) 0;
    set bhInfo($win,d) $mesg
    bind $win <Enter> +[namespace code {balloonhelp_show %W 1}]
    bind $win <Leave> +[namespace code {wm withdraw .balloonhelp;\
					    .balloonhelp.info configure -text ""}]
  }

  proc balloonhelp_pending {win} {
    variable bhInfo
    variable delay
    variable enable
    balloonhelp_cancel
    if {! $enable} {return}
    if {[info exist bhInfo($win,d)]} {
      set PendTime $delay
    } else {
      set PendTime [expr {$delay / 2}]
    }
    set bhInfo(pending) [after $PendTime [namespace code [list balloonhelp_show $win 0]]]
  }

  proc balloonhelp_cancel {} {
    variable bhInfo
    variable waitingForGetDisplays
    if { [info exists bhInfo(pending)]} {
      after cancel $bhInfo(pending)
      unset bhInfo(pending)
    }
    wm withdraw .balloonhelp
    .balloonhelp.info configure -text ""
    if {[info exists waitingForGetDisplays] } {
      set ::displays::abortDisplayProbe 1
    }
  }

  proc balloonhelp_show {win d} {
    variable enable
    variable bhInfo
    variable bhOverlapP 

    if {[winfo exists $win]} {
      if {$enable} {
	if {$d} {
	  set Text $bhInfo($win,d)
	} else {
	  set Text $bhInfo($win)
	}
	#      puts "$Text"
	if { [string range $Text 0 1] == "\[_" } {
	  set Text [eval concat $Text]
	}
	.balloonhelp.info configure -text $Text;
	# Need the update to allow the wm to do its thing... before we ask
	# how big the window is/will be.
	set mx [winfo pointerx .] 
	set my [winfo pointery .]
	#
	# Least I forget, one of the window managers does not actually size the 
	# window until it is up, thus we raise it and then move it if needed.
	# (May cause a slight flash of the first position (below and left of ptr.)
	#
	# Since it is not really possible to get display boundries in multi-display
	# screens, we will position the balloonhelp window as close to the center
	# of the window containg the pointer with one of the four corners at the 
	# pointer position, unless that would put the window off screen (screen not display).
	#

	# the +5 below can push us out of the window
	set w [winfo containing $mx $my]
	incr mx 5
	incr my 5
	while {[set pw [winfo parent $w]] != {} } {set w $pw}
	lassign [split [winfo geometry $w] "+x"] cw ch cx cy
	wm geometry .balloonhelp +$mx+$my
	#puts "[winfo geometry $w] $cw< $ch< $cx< $cy<"
	wm deiconify .balloonhelp
	raise .balloonhelp
	update
	set sz "[regsub -all {[x+]} [winfo geometry .balloonhelp] { }]"
	#puts "$sz"
	lassign $sz mw mh 
	#      puts "$Boundries $mx $my"
	# The players: m - my window (the help)
	#              c - the containing window (may be negative x or y)
	#              p - other possible position 
	#              s - screen
	set sh [winfo screenheight $w]
	set sw [winfo screenwidth $w]

	set px [expr {$mx - $mw -10}]
	set py [expr {$my - $mh -10}]

	if {$mx > $cx + $cw / 2 || $mx + $mw > $sw && $px > 0} {
	  set mx $px
	}
	if {$my > $cy + $ch / 2  || $my + $mh > $sh && $py > 0} {
	  set my $py
	}
	wm geometry .balloonhelp +$mx+$my
      }
    }
    if {[info exist bhInfo(pending)]} {
      unset bhInfo(pending)
    }
  }
}
#if 0
#proc ControlBalloonHelp {} {
#    if {$::BalloonHelpToggle} {
#	balloonhelp_control 1
#    } else {
#	balloonhelp_control 0
#    }
#}

#set BalloonHelpToggle 1
#ControlBalloonHelp;
#endif
