#
# EntryDialog    put up a window and get a responce of
#                text and possibly checkboxes
#
#  parms:
#  wm            the master window if not set will use parent
#  wm_title      title for the window and icon
#  info_text     A message to display below the input area
#  start_entry   The starting string in the input area
#  icon          A bit map to display in the window (optional)
#  args          A list of configuration pairs {item value} (optional)
#                If an 'item' starts with "-" it is taken as a config
#                item for the entry widget. This accomidates fonts
#                colors, textvariables, etc.  If the 'item' does not
#                start with a "-" it is a label for a checkbutton
#                and the value is the variable entry for a checkbutton.
#                If the 'item' starts with a '+' then 'item' (less the '+')
#                is taken as a label for a button and 'value' is taken
#                as a command script for that button.
#                There may be any number of arguments each of which is
#                a list of entries

proc EntryDialog { wm wm_title info_text start_entry \
		       {icon ""}   args} {
#  global fonts colours glob

  set w .entry_dialog[incr ::entry_dialog_count]
  set e $::entry_dialog_count
  toplevel $w -class Dialog
  wm title $w $wm_title
  wm protocol $w WM_DELETE_WINDOW "set ::entry_dialog_return$e {}  \; destroy $w"
  wm iconname $w $wm_title
  wm resizable $w true false
  if { $wm == "" } {
    set wt [winfo toplevel [winfo parent $w]]
    set wp $w
  } else {
    set wt [set wp $wm]
  }
  wm transient $w $wt
  bind $wt <Destroy> "+set ::entry_dialog_return$e {}  \; destroy $w"
# Try this... Duh, not useful.
 # wm attributes $w -topmost

  frame $w.bot
  entry $w.entry -highlightthickness 1 \
      -width 70 \
      -textvariable ::entry_dialog_return$e
  $w.entry delete 0 end
  set ::entry_dialog_return$e $start_entry
  $w.entry xview end

  set text_length [string length $info_text]
  set info_text [string range $info_text 0 1000]
  if {$text_length > [string length $info_text]} {
    set info_text [_ "%s\n\n...etc..." $info_text]
  }
  label $w.bot.info_text -justify left -text "$info_text"  -wraplength 5i

  button $w.bot.ok -text [_ "OK"] -command "destroy $w"
  button $w.bot.cancel -text [_ "Cancel"] \
      -command "set ::entry_dialog_return$e {} \n  destroy $w"

  pack $w.bot -side bottom -expand 1 -fill x
  pack $w.bot.cancel -side right -anchor s
  pack $w.bot.ok -side right -anchor s

  foreach arg $args {
    foreach {label var} $arg {
#      puts "$label $var"
      switch -glob $label {
	-* {set r [catch "$w.entry config $label \"$var\"" ho]}
	+* {
	  set label [string range 1 end $label]
	  button $w.bot.$label -text $label -command $var
	  pack $w.bot.$label  -side right -anchor s
	}
	default {
	  checkbutton $w.bot.$label -variable $var -text $label
	  pack $w.bot.$label -side right -anchor s
	}
      }
    }
  }
  if {$icon != ""} {
    label $w.bot.icon -bitmap $icon 
    pack $w.bot.icon -side left -padx 20 -anchor n -pady 2
  }
  pack $w.bot.info_text -side left -fill x -expand 1 -anchor w

#-padx 8 -pady 5

  pack $w.entry -side bottom -padx 8 -pady 8 -expand 1 -fill x

#  set glob(entry_dialog_return) {}

  bind $w.entry <Return> " destroy $w"
  

  bind $w.entry <KP_Enter> "destroy $w"
  
#    set glob(abortcmd) 1
  
  bind $w.entry <Escape> "
    set ::entry_dialog_return$e {}
    destroy $w
  "
  $w.entry icursor end
  set oldFocus [focus]
  if {[set oldGrab [grab current $w]] != ""} {
    # Don't really know why this is needed, but focus does not
    # move without it.
    event generate $oldFocus <Leave>
  }
  update
  centerWin $w $wt
  grab $w
  focus $w.entry
#  wm withdraw $w
#  wm deiconify $w
#  puts " $w.entry"
  tkwait window $w
  catch {focus $oldFocus}
  if {$oldGrab != ""} {
    grab $oldGrab
  }
  return [set ::entry_dialog_return$e]
}
