#
# This set of routines is ment to be called to set up a
# search capability for text windows.  It is to be called
# after the text window is created with the window name
# and a title for the tearoff menu which is raised with
# a button <3> mouse click.  In addition to the three
# entries in the menu, the caller can supply pre and post
# entries to be put in the menu.  
# The supplied entries should be in a list
# with a 'label' 'script' pairs.  An empty 'script' 
# i.e. '{}' will generate a 'separator' entry.
#
# textSearch   sets up the <3> menu for
#
#  wtext           the text window name
#  title           the title for the sub menu
#  config          The name of a list of value pairs to configure the window
#                  these should be -item value pairs (optional)
#                  A -item that ends in "tag" will be used to configure
#                  the tag used to highlight the found search.
#                  If config starts with "+" it indicats a script
#                  that will be executed to get the config list.
#                  This is passed by name so that it can be configured
#                  after the widows are built.
#  preMenuEntries  a list of menu item pairs {label command} (optional)
#  postMenuEntries a list of menu item pairs {label command} (optional)



proc textSearch { wtext title {config {}} \
		      {preMenuEntries {}}  {postMenuEntries {}}} {
  set w [regsub {(.*)\.[^\.]+$} $wtext {\1}]
  set tl [regsub {(^\.[^\.]+).*} $wtext {\1}]
  menu $wtext.p -tearoffcommand "ReplaceTearoff $w" -title "$title"
  foreach {label cmd} $preMenuEntries {
    if {$cmd == {} } {
      $wtext.p add separator
    } else {
      $wtext.p add command -label $label -command $cmd
    }
  }
  foreach {item value} [searchViewGetConfig $config] {
    switch -regexp -matchvar subitem $item {
      {(.*)menu$} {
	catch "$wtext.p config [lindex $subitem 1] \"$value\""
      }
    }
  }
  $wtext.p add command -label [_ "Search..."] \
      -command "SearchView $wtext [list $config] 0"
  $wtext.p add command -label [_ "Search Again Forward"] \
      -command "SearchView $wtext [list $config] 1"
  $wtext.p add command -label [_ "Search Again Backward"] \
      -command "SearchView $wtext [list $config] 2"
  foreach {label cmd} $postMenuEntries {
    if {$cmd == {} } {
      $wtext.p add separator
    } else {
      $wtext.p add command -label $label -command $cmd
    }
  }
  bind $wtext <3> "tk_popup $wtext.p %X %Y"
  bind $w <Escape> "destroy $w"
  bind $w <Next> "$wtext yview scroll 1 pages"
  bind $w <Prior> "$wtext yview scroll -1 pages"
  bind $w <1> "bind $w \<Next\> \"\" ; bind $w \<Prior\> \"\""
  bind $w <Home> "$wtext see 0.0"
  bind $w <End> "$wtext see end"
  bind $w <Alt-Escape> {break}
#  bind $tl <Destroy> "+destroy .entry_dialog"
  return $w
}

proc searchViewGetConfig {config} {
  if {$config == {} } {return {}}
  if {[string range $config 0 0] == "+"} {
    return [eval [string range $config 1 end]]
  } else {
    return [set $config]
  }
}

proc SearchView { w config again } {
  global glob colours
  incr glob(search,regexp) 0
  incr glob(search,casefold) 0
  incr glob(search,reverse) 0
  incr glob(search,fromhere) 0
  incr glob(search,len) 0
  if {![info exists glob(searchstring)]} {
    set glob(searchstring) ""
  }
  destroy .apop
  set tag select
  set pconfig {}

  $w tag config $tag -background [lindex [$w config -selectbackground] 4]
  $w tag config $tag -foreground [lindex [$w config -selectforeground] 4]

  foreach {item value} [searchViewGetConfig $config] {
    switch -regexp -matchvar subitem $item {
      {(.*)tag$} {
	$w tag config $tag [lindex $subitem 1] $value
#	puts "$w  [lindex $subitem 1] $value"
      }
      default { lappend pconfig $item $value}
    }
  }
  if {$again == 0} {
    set s [EntryDialog $w [_ "Search..."]\
	       [_ "Enter text to search for"] $glob(searchstring) question \
	       [list \
		    "regexp" glob(search,regexp)\
		    "casefold" glob(search,casefold)\
		    "reverse" glob(search,reverse)\
		    "from here" glob(search,fromhere)] \
	       $pconfig ]
    if {$s == ""} return
    set glob(searchstring) $s
    if {! $glob(search,fromhere)} {
      $w mark set insert 0.0
    }
    set reverse [expr {$glob(search,reverse) ? "backwards" : "forwards"}]
  } else {

    switch $again {
      2 {
	set reverse "backwards"
	if {[$w compare insert <= 0.0]} {
	  $w mark set insert end
	}
      }
      1 -
      default {
	set reverse "forwards"
	if {[$w compare insert == end-1c]} {
	  $w mark set insert 0.0
	}
      }
    }
    if { $reverse == "backwards"} {
    } else {
      $w mark set insert "[$w index insert] + $glob(search,len) chars"
    }
  }
  set regexp [expr {$glob(search,regexp) ? "regexp" : "exact"}]
  set case [expr {$glob(search,casefold) ? "nocase" : "elide"}]
  set stopIndex [expr { $reverse == "backwards" ? "0.0" : "end" }]
	      
  $w tag remove $tag 0.0 end
  set idx [$w search -count glob(search,len) -$case -$reverse -$regexp -- \
	       $glob(searchstring) insert $stopIndex]
  if {$idx == ""} {
    $w mark set insert [expr {$reverse == "backwards" ? "0.0" : "end"}]
    smart_dialog .apop $w {Info} [list {} $glob(searchstring) " not found"] \
	0 0 {} $pconfig
#    PopInfo [_ "%s not found" $glob(searchstring)]
    return
  }
  $w mark set insert  $idx
  $w tag add $tag $idx "$idx + $glob(search,len) chars"
#  puts "$idx $len"
  $w see insert
}
